import React, { useState, useEffect } from 'react';
import { Layout, Menu, MenuProps, theme } from 'antd';
import { Routes, Route, Link, useNavigate } from 'react-router-dom';
import {
  MenuFoldOutlined,
  MenuUnfoldOutlined,
  InboxOutlined,
  FileOutlined,
  UserOutlined,
  SettingOutlined,
  LogoutOutlined,
  EnvironmentOutlined,
} from '@ant-design/icons';
import CategoryManagement from './admin/CategoryManagement';
import UserManagement from './admin/UserManagement';
import RoleManagement from './admin/RoleManagement';
import ReportGenerator from './reports/ReportGenerator';
import CategoryItems from './inventory/CategoryItems';
import PrivateRoute from './PrivateRoute';
import LocationManagement from './admin/LocationManagement';

const { Header, Sider, Content } = Layout;

const Dashboard: React.FC = () => {
  const [collapsed, setCollapsed] = useState(false);
  const { token: { colorBgContainer } } = theme.useToken();
  const [userRole, setUserRole] = useState<string | null>(null);
  const navigate = useNavigate();

  // ✅ Получаем роль пользователя через локальное хранилище
  useEffect(() => {
    const user = localStorage.getItem('user');
    const parsedUser = user ? JSON.parse(user) : null;
    setUserRole(parsedUser?.role || null);
  }, []);

  const handleLogout = () => {
    localStorage.removeItem('token');
    localStorage.removeItem('user');
    navigate('/login');
  };

  const menuItems: MenuProps['items'] = [
    {
      key: '1',
      icon: <InboxOutlined />,
      label: <Link to="/dashboard/inventory">Инвентарь</Link>,
    },
    {
      key: '2',
      icon: <FileOutlined />,
      label: <Link to="/dashboard/reports">Отчеты</Link>,
    },
  ];
  
  // ✅ Добавляем "Администрирование", если роль admin
  if (userRole === 'admin') {
    menuItems.push({
      key: 'admin',
      icon: <SettingOutlined />,
      label: collapsed ? <SettingOutlined /> : 'Управление', // ✅ Фикс стрелки
      children: [
        {
          key: '3',
          icon: <UserOutlined />,
          label: <Link to="/dashboard/admin/users">Пользователи</Link>,
        },
        {
          key: '4',
          icon: <UserOutlined />,
          label: <Link to="/dashboard/admin/roles">Роли</Link>,
        },
        {
          key: '5',
          icon: <InboxOutlined />,
          label: <Link to="/dashboard/admin/categories">Категории</Link>,
        },
        {
          key: '6',
          icon: <EnvironmentOutlined />,
          label: <Link to="/dashboard/admin/locations">Локации</Link>,
        },
      ],
    });
  }

  // ✅ Добавляем кнопку "Выйти" в `menuItems`, чтобы она скрывалась при сворачивании
  menuItems.push({
    key: 'logout',
    icon: <LogoutOutlined />,
    label: <Link to={''} >Выйти</Link>,
    onClick: handleLogout, // ✅ Добавляем обработчик выхода
  });

  return (
    <Layout style={{ minHeight: '100vh' }}>
      <Sider trigger={null} collapsible collapsed={collapsed}>
        <div style={{ height: 32, margin: 16, background: 'rgba(255, 255, 255, 0.2)' }} />
        <Menu theme="dark" mode="inline" defaultSelectedKeys={['1']} items={menuItems} />
      </Sider>
      <Layout>
        <Header style={{ padding: 0, background: colorBgContainer }}>
          {React.createElement(collapsed ? MenuUnfoldOutlined : MenuFoldOutlined, {
            className: 'trigger',
            onClick: () => setCollapsed(!collapsed),
            style: { fontSize: '18px', padding: '0 24px', cursor: 'pointer' },
          })}
        </Header>
        <Content style={{ margin: '24px 16px', padding: 24, background: colorBgContainer }}>
          <Routes>
            <Route path="/inventory" element={<CategoryItems categoryId={1} />} />
            <Route path="/reports" element={<ReportGenerator />} />
            <Route
              path="/admin/users"
              element={
                <PrivateRoute requiredRoles={['admin']}>
                  <UserManagement />
                </PrivateRoute>
              }
            />
            <Route
              path="/admin/roles"
              element={
                <PrivateRoute requiredRoles={['admin']}>
                  <RoleManagement />
                </PrivateRoute>
              }
            />
            <Route
              path="/admin/categories"
              element={
                <PrivateRoute requiredRoles={['admin']}>
                  <CategoryManagement />
                </PrivateRoute>
              }
            />
            <Route
              path="/admin/locations"
              element={
                <PrivateRoute requiredRoles={['admin']}>
                  <LocationManagement />
                </PrivateRoute>
              }
            />
          </Routes>
        </Content>
      </Layout>
    </Layout>
  );
};

export default Dashboard;
