import React, { useState, useEffect } from 'react';
import { Table, Button, Modal, Form, Input, Checkbox, Space, App as AntdApp } from 'antd';
import { useGet, usePost, usePut, useDelete } from '../../hooks/useApi';

interface Permission {
  id: number;
  name: string;
  description: string;
}

interface Role {
  id: number;
  name: string;
  description: string;
  permissions: Permission[];
}

const RoleManagement: React.FC = () => {
  const [isModalVisible, setIsModalVisible] = useState(false);
  const [editingRole, setEditingRole] = useState<Role | null>(null);
  const [form] = Form.useForm();
  const [loading, setLoading] = useState(false);
  const { message } = AntdApp.useApp();
  // Используем `useGet` для загрузки ролей и разрешений
  const { data: roles, execute: fetchRoles } = useGet<Role[]>('/admin/roles');
  const { data: permissions, execute: fetchPermissions } = useGet<Permission[]>('/admin/permissions');

  // Используем `usePost`, `usePut` и `useDelete` для работы с ролями
  const { execute: createRole } = usePost('/admin/roles');
  const { execute: updateRole } = usePut(`/admin/roles/${editingRole?.id}`);
  const { execute: deleteRole } = useDelete(`/admin/roles`);

  // Загружаем данные при монтировании компонента
  useEffect(() => {
    fetchRoles();
    fetchPermissions();
  }, []);

  // Открытие модального окна для редактирования роли
  const handleEdit = (role: Role) => {
    setEditingRole(role);
    form.setFieldsValue({
      name: role.name,
      description: role.description,
      permissions: role.permissions.map(p => p.id),
    });
    setIsModalVisible(true);
  };

  // Отправка формы (создание или обновление роли)
  const handleSubmit = async (values: any) => {
    setLoading(true);
    try {
      if (editingRole) {
        await updateRole(values);
        message.success('Роль обновлена');
      } else {
        await createRole(values);
        message.success('Роль создана');
      }
      setIsModalVisible(false);
      form.resetFields();
      setEditingRole(null);
      fetchRoles(); // Обновляем список ролей
    } catch {
      message.error('Ошибка при сохранении роли');
    } finally {
      setLoading(false);
    }
  };

  // Удаление роли
  const handleDelete = async (roleId: number) => {
    try {
      await deleteRole(roleId);
      message.success('Роль удалена');
      fetchRoles(); // Обновляем список ролей
    } catch {
      message.error('Ошибка при удалении роли');
    }
  };

  // Определение столбцов таблицы
  const columns = [
    {
      title: 'Название',
      dataIndex: 'name',
      key: 'name',
    },
    {
      title: 'Описание',
      dataIndex: 'description',
      key: 'description',
    },
    {
      title: 'Разрешения',
      dataIndex: 'permissions',
      key: 'permissions',
      render: (perms: Permission[]) => perms.map(p => p.name).join(', '),
    },
    {
      title: 'Действия',
      key: 'actions',
      render: (_: any, record: Role) => (
        <Space>
          <Button type="link" onClick={() => handleEdit(record)}>
            Редактировать
          </Button>
          <Button 
            type="link" 
            danger 
            onClick={() => handleDelete(record.id)}
            disabled={record.name === 'admin'} // Защита от удаления роли админа
          >
            Удалить
          </Button>
        </Space>
      ),
    },
  ];

  return (
    <div>
      <Button
        type="primary"
        onClick={() => {
          setEditingRole(null);
          form.resetFields();
          setIsModalVisible(true);
        }}
        style={{ marginBottom: 16 }}
      >
        Создать роль
      </Button>

      <Table
        columns={columns}
        dataSource={roles || []}
        rowKey="id"
        loading={!roles}
      />

      <Modal
        title={editingRole ? 'Редактировать роль' : 'Создать роль'}
        open={isModalVisible}
        onOk={() => form.submit()}
        onCancel={() => {
          setIsModalVisible(false);
          setEditingRole(null);
          form.resetFields();
        }}
        confirmLoading={loading}
      >
        <Form
          form={form}
          onFinish={handleSubmit}
          layout="vertical"
        >
          <Form.Item
            name="name"
            label="Название"
            rules={[{ required: true, message: 'Введите название роли' }]}
          >
            <Input disabled={editingRole?.name === 'admin'} />
          </Form.Item>

          <Form.Item
            name="description"
            label="Описание"
          >
            <Input.TextArea />
          </Form.Item>

          <Form.Item
            name="permissions"
            label="Разрешения"
          >
            <Checkbox.Group>
              {(permissions || []).map(permission => (
                <div key={permission.id} style={{ marginBottom: 8 }}>
                  <Checkbox
                    value={permission.id}
                    disabled={editingRole?.name === 'admin'}
                  >
                    {permission.description}
                  </Checkbox>
                </div>
              ))}
            </Checkbox.Group>
          </Form.Item>
        </Form>
      </Modal>
    </div>
  );
};

export default RoleManagement;
