import React, { useState, useEffect } from 'react';
import { Table, Button, Modal, Form, Input, Select, Switch, App as AntdApp } from 'antd';
import { useGet, usePost, usePatch } from '../../hooks/useApi';

interface User {
  id: number;
  username: string;
  role: string;
  is_active: boolean;
  categories: string[];
}

interface Role {
  id: number;
  name: string;
}

interface Category {
  id: number;
  name: string;
}

const UserManagement: React.FC = () => {
  const [isModalVisible, setIsModalVisible] = useState(false);
  const [form] = Form.useForm();
  const [loading, setLoading] = useState(false);
  const { message } = AntdApp.useApp();
  // Используем `useGet` для загрузки пользователей, ролей и категорий
  const { data: users, execute: fetchUsers } = useGet<User[]>('/admin/users');
  const { data: roles, execute: fetchRoles } = useGet<Role[]>('/admin/roles');
  const { data: categories, execute: fetchCategories } = useGet<Category[]>('/admin/categories');

  // Используем `usePost` для создания пользователя
  const { execute: createUser } = usePost('/admin/users');

  // Используем `usePatch` для обновления статуса пользователя
  const { execute: updateUserStatus } = usePatch(`/admin/users`);

  // Загружаем данные при монтировании компонента
  useEffect(() => {
    fetchUsers();
    fetchRoles();
    fetchCategories();
  }, []);

  // Обработчик создания пользователя
  const handleCreate = async (values: any) => {
    setLoading(true);
    try {
      await createUser(values);
      message.success('Пользователь создан');
      setIsModalVisible(false);
      form.resetFields();
      fetchUsers();
    } catch {
      message.error('Ошибка при создании пользователя');
    } finally {
      setLoading(false);
    }
  };

  // Обработчик изменения статуса пользователя (активен/не активен)
  const handleToggleActive = async (userId: number, isActive: boolean) => {
    try {
      await updateUserStatus(`${userId}`, { is_active: isActive });
      message.success('Статус пользователя обновлен');
      fetchUsers();
    } catch {
      message.error('Ошибка при обновлении статуса');
    }
  };

  // Определение столбцов таблицы
  const columns = [
    {
      title: 'Имя пользователя',
      dataIndex: 'username',
      key: 'username',
    },
    {
      title: 'Роль',
      dataIndex: 'role',
      key: 'role',
    },
    {
      title: 'Активен',
      dataIndex: 'is_active',
      key: 'is_active',
      render: (active: boolean, record: User) => (
        <Switch
          checked={active}
          onChange={(checked) => handleToggleActive(record.id, checked)}
        />
      ),
    },
    {
      title: 'Категории',
      dataIndex: 'categories',
      key: 'categories',
      render: (categories: string[]) => categories.join(', '),
    },
  ];

  return (
    <div>
      <Button
        type="primary"
        onClick={() => setIsModalVisible(true)}
        style={{ marginBottom: 16 }}
      >
        Создать пользователя
      </Button>

      <Table
        columns={columns}
        dataSource={users || []}
        rowKey="id"
        loading={!users}
      />

      <Modal
        title="Создать пользователя"
        open={isModalVisible}
        onOk={() => form.submit()}
        onCancel={() => setIsModalVisible(false)}
        confirmLoading={loading}
      >
        <Form
          form={form}
          onFinish={handleCreate}
          layout="vertical"
        >
          <Form.Item
            name="username"
            label="Имя пользователя"
            rules={[{ required: true }]}
          >
            <Input />
          </Form.Item>

          <Form.Item
            name="password"
            label="Пароль"
            rules={[{ required: true }]}
          >
            <Input.Password />
          </Form.Item>

          <Form.Item
            name="role_id"
            label="Роль"
            rules={[{ required: true }]}
          >
            <Select>
              {(roles || []).map((role) => (
                <Select.Option key={role.id} value={role.id}>
                  {role.name}
                </Select.Option>
              ))}
            </Select>
          </Form.Item>

          <Form.Item
            name="categories"
            label="Доступные категории"
          >
            <Select mode="multiple">
              {(categories || []).map((category) => (
                <Select.Option key={category.id} value={category.id}>
                  {category.name}
                </Select.Option>
              ))}
            </Select>
          </Form.Item>
        </Form>
      </Modal>
    </div>
  );
};

export default UserManagement;
