import React, { useState, useEffect } from 'react';
import { Card, Form, Select, Button, Radio, App as AntdApp } from 'antd';
import { DownloadOutlined } from '@ant-design/icons';
import { useGet, useApi } from '../../hooks/useApi';

interface Location {
  id: number;
  name: string;
}

interface Category {
  id: number;
  name: string;
  location_id: number;
}

interface FormValues {
  reportType: 'inventory' | 'boxes';
  format: 'excel' | 'pdf';
  locationId?: number;
  categoryId?: number;
}

const ReportGenerator: React.FC = () => {
  const [selectedLocation, setSelectedLocation] = useState<number | undefined>();
  const [loading, setLoading] = useState(false);
  const { message } = AntdApp.useApp();

  // Используем `useGet` для загрузки локаций
  const { data: locations, execute: fetchLocations } = useGet<Location[]>('/locations');
  const { data: categories, execute: fetchCategories } = useGet<Category[]>('');

  // API-запрос для скачивания отчётов
  const reportApi = useApi<Blob>('', 'GET');

  // Загружаем локации при монтировании компонента
  useEffect(() => {
    fetchLocations();
  }, []);

  useEffect(() => {
    if (selectedLocation) {
      fetchCategories(`/locations/${selectedLocation}/categories`);
    } else {
      fetchCategories('');
    }
  }, [selectedLocation]);

  // Обработчик скачивания отчёта
  const handleDownload = async (values: FormValues) => {
    const { reportType, format, locationId, categoryId } = values;
    setLoading(true);

    try {
      let url = '/reports/';
      if (reportType === 'inventory') {
        url += `inventory?format=${format}`;
        if (locationId) url += `&locationId=${locationId}`;
        if (categoryId) url += `&categoryId=${categoryId}`;
      } else {
        if (!categoryId) {
          message.error('Выберите категорию для отчета по коробкам');
          setLoading(false);
          return;
        }
        url += `boxes/${categoryId}?format=${format}`;
      }

      // Выполняем запрос на скачивание отчета
      const response = await reportApi.execute(url, {
        headers: { 'Content-Type': 'application/json' },
        responseType: 'blob', // Важно для скачивания файла
        timeout: 30000, // Долгий таймаут для больших отчетов
      });

      if (!response) {
        message.error('Ошибка при генерации отчета');
        return;
      }

      // Создаем Blob и скачиваем файл
      const blob = new Blob([response], {
        type:
          format === 'excel'
            ? 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
            : 'application/pdf',
      });

      const downloadUrl = window.URL.createObjectURL(blob);
      const link = document.createElement('a');
      link.href = downloadUrl;
      link.download = `report-${new Date().toISOString()}.${format}`;
      document.body.appendChild(link);
      link.click();
      link.remove();
      window.URL.revokeObjectURL(downloadUrl);

      message.success('Отчет успешно сгенерирован');
    } catch {
      message.error('Ошибка при генерации отчета');
    } finally {
      setLoading(false);
    }
  };

  return (
    <Card title="Генерация отчетов">
      <Form<FormValues> layout="vertical" onFinish={handleDownload}>
        <Form.Item
          name="reportType"
          label="Тип отчета"
          rules={[{ required: true, message: 'Выберите тип отчета' }]}
        >
          <Radio.Group>
            <Radio.Button value="inventory">Инвентарный отчет</Radio.Button>
            <Radio.Button value="boxes">Отчет по коробкам</Radio.Button>
          </Radio.Group>
        </Form.Item>

        <Form.Item
          name="format"
          label="Формат"
          rules={[{ required: true, message: 'Выберите формат' }]}
        >
          <Radio.Group>
            <Radio.Button value="excel">Excel</Radio.Button>
            <Radio.Button value="pdf">PDF</Radio.Button>
          </Radio.Group>
        </Form.Item>

        <Form.Item name="locationId" label="Локация">
          <Select
            placeholder="Выберите локацию"
            onChange={(value) => setSelectedLocation(value)}
            allowClear
          >
            {(locations || []).map((location) => (
              <Select.Option key={location.id} value={location.id}>
                {location.name}
              </Select.Option>
            ))}
          </Select>
        </Form.Item>

        <Form.Item name="categoryId" label="Категория">
          <Select placeholder="Выберите категорию" allowClear disabled={!selectedLocation}>
            {(categories || []).map((category) => (
              <Select.Option key={category.id} value={category.id}>
                {category.name}
              </Select.Option>
            ))}
          </Select>
        </Form.Item>

        <Form.Item>
          <Button type="primary" icon={<DownloadOutlined />} loading={loading} htmlType="submit">
            Скачать отчет
          </Button>
        </Form.Item>
      </Form>
    </Card>
  );
};

export default ReportGenerator;
