import { useState, useCallback } from 'react';
import axios from 'axios';

const API_URL = import.meta.env.VITE_API_URL || '/api';

interface ApiError {
  message: string;
}

interface UseApiResponse<T> {
  data: T | null;
  loading: boolean;
  error: string | null;
  execute: (...args: any[]) => Promise<T | null>;
}

export function useApi<T>(
  endpoint: string,
  method: 'GET' | 'POST' | 'PUT' | 'PATCH' | 'DELETE' = 'GET'
): UseApiResponse<T> {
  const [data, setData] = useState<T | null>(null);
  const [loading, setLoading] = useState<boolean>(false);
  const [error, setError] = useState<string | null>(null);

  const execute = useCallback(
    async (...args: any[]): Promise<T | null> => {
      try {
        setLoading(true);
        setError(null);

        const token = localStorage.getItem('token');
        const headers = token ? { Authorization: `Bearer ${token}` } : {};

        let response;
        switch (method) {
          case 'GET':
            response = await axios.get<T>(`${API_URL}${endpoint}`, { headers });
            break;
          case 'POST':
            response = await axios.post<T>(`${API_URL}${endpoint}`, args[0], { headers });
            break;
          case 'PUT':
            response = await axios.put<T>(`${API_URL}${endpoint}`, args[0], { headers });
            break;
          case 'PATCH': // Добавляем поддержку PATCH-запросов
            response = await axios.patch<T>(`${API_URL}${endpoint}`, args[0], { headers });
            break;
          case 'DELETE':
            response = await axios.delete<T>(`${API_URL}${endpoint}`, { headers });
            break;
          default:
            throw new Error(`Неподдерживаемый метод: ${method}`);
        }

        setData(response.data);
        return response.data;
      } catch (err: any) {
        if (err && err.isAxiosError) {
          const apiError = err.response?.data as ApiError | undefined;
          const errorMessage = apiError?.message || 'Произошла ошибка на сервере';
          setError(errorMessage);
        } else {
          setError('Произошла неизвестная ошибка');
        }
        return null;
      } finally {
        setLoading(false);
      }
    },
    [endpoint, method]
  );

  return { data, loading, error, execute };
}

// Вспомогательные функции для типичных запросов
export function useGet<T>(endpoint: string) {
  return useApi<T>(endpoint, 'GET');
}

export function usePost<T>(endpoint: string) {
  return useApi<T>(endpoint, 'POST');
}

export function usePut<T>(endpoint: string) {
  return useApi<T>(endpoint, 'PUT');
}

export function usePatch<T>(endpoint: string) {
  return useApi<T>(endpoint, 'PATCH');
}

export function useDelete<T>(endpoint: string) {
  return useApi<T>(endpoint, 'DELETE');
}

export default useApi;
